import josx.platform.rcx.*;

public class LightSensorListener implements SensorListener, Constants {
	
	private static boolean bucketTransport = false;
	
	private static byte round = 1;
	
	private static byte greenCount = 0;

	public static byte borderBG;

	public static byte borderGW;

	private static Sensor sensor;

	private LineSeek lineSeek;

	private static boolean enabled = true;

	private static boolean turning;
	
	public static void init(Sensor sensor) {

		sensor.setTypeAndMode(3, 0x80);
		sensor.activate();

		LightSensorListener.sensor = sensor;

		try {
			initColors();
		} catch (InterruptedException e) {
		}

	}

	private static void initColors() throws InterruptedException {

		Thread.sleep(500);

		LCD.showProgramNumber(WHITE);

		while (!Button.PRGM.isPressed()) {
			LCD.showNumber(sensor.readValue());
		}

		byte white = (byte) sensor.readValue();

		
		Thread.sleep(500);

		LCD.showProgramNumber(GREEN);

		while (!Button.PRGM.isPressed()) {
			LCD.showNumber(sensor.readValue());
		}

		byte green = (byte) sensor.readValue();

		
		Thread.sleep(500);

		LCD.showProgramNumber(BLACK);

		while (!Button.PRGM.isPressed()) {
			LCD.showNumber(sensor.readValue());
		}

		byte black = (byte) sensor.readValue();

		setColors(black, green, white);

	}

	public static void setColors(byte black, byte green, byte white) {

		borderBG = (byte) (green - 2);

		borderGW = (byte) ((green + white) / 2);

	}

	public LightSensorListener(Thread mainThread) {

		lineSeek = new LineSeek(mainThread);
		lineSeek.start();
		
		sensor.addSensorListener(this);

	}

	public static byte getColor(byte val) {

		return val < borderBG ? BLACK : WHITE;

	}

	public static byte getColor() {

		byte val = (byte) sensor.readValue();

		return getColor(val);

	}

	public synchronized void stateChanged(Sensor source, int oldValue, int newValue) {
		
		if (!enabled) return;

		byte color = getColor((byte)newValue);
		
		if (color == BLACK) {
			
			turning = false;	
			
			if (lineSeek.isSeeking()) {
				lineSeek.stopSeeking();
			}
			
		} else { // color == WHITE 
			
			if (!turning && !lineSeek.isSeeking()) {
				lineSeek.startSeeking();
			}
		
		}
		
	}

	public static void greenFound() {
		
		greenCount++;
		
		if (bucketTransport) {
			
			if (greenCount == round) {
				
				ProducerApplication.move(BACKWARD);
				
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {}
				
				ProducerApplication.move(LEFT);
				
				try {
					Thread.sleep(500);
				} catch (InterruptedException e) {}
				
				turning	= true;
				
				greenCount = 0;
				
				bucketTransport = false;
				
				notifyBert();
				
				round++;
				
				if (round == 4) {
					
					ProducerApplication.move(STOP);
					
					while(Serial.isSending());
					Sound.beepSequence();
					AbstractApplication.mainThread.interrupt();
					
				}
				
			} else {
				
				ProducerApplication.move(FORWARD);
				
				try {
					Thread.sleep(200);
				} catch (InterruptedException e) {}
				
				if (round > 1 && greenCount == round - 1) {
					ProducerApplication.move(STOP);
					ProducerApplication.waitForBert();
					ProducerApplication.move(FORWARD);
				}
			
			}			
			
		} else {
			
			if (greenCount == round) {
				
				ProducerApplication.move(FORWARD);
				
				try {
					Thread.sleep(800);
				} catch (InterruptedException e) {}
				
				ProducerApplication.move(LEFT);
				
				try {
					Thread.sleep(500);
				} catch (InterruptedException e) {}
				
				turning = true;
				
				greenCount = 0;
				
				bucketTransport = true;
				
			} else {
				
				ProducerApplication.move(FORWARD);
				
				try {
					Thread.sleep(200);
				} catch (InterruptedException e) {}
				
			}				
			
		}
	
		LCD.showNumber(round);
		LCD.showProgramNumber(bucketTransport ? 1 : 0);
		
	}

	private static void notifyBert() {
		
		byte[] data = new byte[2];
		
		data[0] = (byte)(data.length - 1); // 2
		data[1] = 23; 
		
		Serial.sendPacket(data,0,data.length);	
		
	}

	public static Sensor getSensor() {
		return sensor;
	}
	
	public static boolean isEnabled() {
		return enabled;
	}

	public static void setEnabled(boolean aoff) {
		enabled = aoff;
	}

	public static byte getRound() {
		return round;
	}

}
